/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QDebug>

#include <QAction>
#include <QButtonGroup>
#include <QClipboard>
#include <QCloseEvent>
#include <QDesktopWidget>
#include <QDir>
#include <QFile>
#include <QFileDialog>
#include <QGridLayout>
#include <QLayout>
#include <QMenu>
#include <QMessageBox>
#include <QMimeData>
#include <QMovie>
#include <QPixmap>
#include <QProcess>
#include <QPushButton>
#include <QSettings>
#include <QSignalMapper>
#include <QTextStream>
#include <QTextDocumentFragment>
#include <QToolButton>
#include <QUrl>

#include "defines.h"
#include "Util.h"
#include "EquationView.h"
#include "DialogReplace.h"
#include "DialogPreferences.h"
#include "DialogAbout.h"
#include "EquationTemplateWidget.h"
#include "FileInfo.h"
#include "FrameMathTab.h"
#include "LatexEditor.h"
#include "WidgetFind.h"
#include "WidgetToolMenu.h"
#include "RenderEngine.h"

#include "MainWindow.h"
#include "ui_mainwindow.h"


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent),
      ui(new Ui::MainWindowClass),
      equationView(0),
      mAutoUpdateTimer()
{
    ui->setupUi(this);

    setWindowTitle(APP_FULL_NAME);

    ui->fgColorPicker->removeItem(1);
    ui->fgColorPicker->setCurrentIndex(1);
    ui->bgColorPicker->setCurrentIndex(1);


    editor = ui->editEquation;
    findWidget = ui->findWidget;

    templateTab = new FrameMathTab;

    replaceDialog = new DialogReplace(this);
    preferencesDialog = new DialogPreferences(this);
    aboutDialog = new DialogAbout(this);

    equation = new EqualX::FileInfo(this);

    equationView = new EquationView(this);
    equationView->setVisible(true);
    equationView->setAcceptDrops(true);

    renderModeButtonGroup = new QButtonGroup;
    renderModeButtonGroup->addButton(ui->renderModeDisplay, LATEX_ENV_MODE_DISPLAY);
    renderModeButtonGroup->addButton(ui->renderModeInline, LATEX_ENV_MODE_INLINE);
    renderModeButtonGroup->addButton(ui->renderModeAlign, LATEX_ENV_MODE_ALIGN);
    renderModeButtonGroup->addButton(ui->renderModeText, LATEX_ENV_MODE_TEXT);

    QStringList treeWidgetErrorsHeaderLabels;
    treeWidgetErrorsHeaderLabels.append(tr("Line"));
    treeWidgetErrorsHeaderLabels.append(tr("Message"));
    ui->treeWidgetErrors->setHeaderLabels(treeWidgetErrorsHeaderLabels);

    // this is the directory holding all files generated by the Render Engine
    QDir buildDir(QDir::tempPath());
    buildDir.mkdir(APP_NAME);
    buildDir.cd(APP_NAME);

    renderer = new EqualX::RenderEngine;
    renderer->setBuildDir(buildDir.absolutePath());

    movieWorkInProgress = new QMovie(":/resources/icons/spinner.gif");
    movieWorkInProgress->setScaledSize(QSize(64,64));

    setConverters();
    setActions();
    setSymbolsPanel();
    setMathPanelTabsActions();
    setSignals();

    ui->editPreamble->hide();
    findWidget->hide();
    editor->setFocus();

    ui->equationArea->setWidget(equationView);
    updateRecentFileActions();

    mAutoUpdateTimer.setSingleShot(true);

    // update export combobox from preferences
    preferencesDialog->exportCBox->addItems(renderer->fileTypes());

    readSettings();
    updateUI();
    checkRequirements();

    newEquation();

    setWindowModified(false);
}

MainWindow::~MainWindow()
{
    delete ui;
    delete renderModeButtonGroup;
    delete movieWorkInProgress;
    delete renderer;
}

void MainWindow::setWindowModified(bool mod)
{
    ui->actionSave->setEnabled(mod);
    ui->actionSaveAs->setEnabled(mod || equationView->pixmap());

    QMainWindow::setWindowModified(mod);
}

void MainWindow::setActions()
{
    /* add actions to Menus for Symbols Toolbar */
    toolDeco = new WidgetToolMenu(2, 1, this);
    toolDeco->addAction(ui->actionDeco_super);
    toolDeco->addAction(ui->actionDeco_under);
    toolDeco->addAction(ui->actionDeco_overset);
    toolDeco->addAction(ui->actionDeco_underset);

    toolFunc= new WidgetToolMenu(2, 1, this);
    toolFunc->addAction(ui->actionFunc_frac);
    toolFunc->addAction(ui->actionFunc_root);
    toolFunc->addAction(ui->actionFunc_fracNoHLine);
    toolFunc->addAction(ui->actionFunc_sqrt);

    toolDelim= new WidgetToolMenu(3, 1, this);
    toolDelim->addAction(ui->actionDelim_paraSimple);
    toolDelim->addAction(ui->actionDelim_squareSimple);
    toolDelim->addAction(ui->actionDelim_ceilSimple);
    toolDelim->addAction(ui->actionDelim_curlySimple);
    toolDelim->addAction(ui->actionDelim_angleSimple);
    toolDelim->addAction(ui->actionDelim_floorSimple);
    toolDelim->addAction(ui->actionDelim_vertSimple);
    toolDelim->addAction(ui->actionDelim_barsSimple);


    toolAccents = new WidgetToolMenu(3, 1, this);
    toolAccents->addAction(ui->actionAccents_hat);
    toolAccents->addAction(ui->actionAccents_acute);
    toolAccents->addAction(ui->actionAccents_breve);
    toolAccents->addAction(ui->actionAccents_tilde);
    toolAccents->addAction(ui->actionAccents_underleftrightarrow);
    toolAccents->addAction(ui->actionAccents_underline);
    toolAccents->addAction(ui->actionAccents_bar);
    toolAccents->addAction(ui->actionAccents_check);
    toolAccents->addAction(ui->actionAccents_dot);
    toolAccents->addAction(ui->actionAccents_dotdouble);
    toolAccents->addAction(ui->actionAccents_grave);
    toolAccents->addAction(ui->actionAccents_hatwide);
    toolAccents->addAction(ui->actionAccents_overbrace);
    toolAccents->addAction(ui->actionAccents_overleftarrow);
    toolAccents->addAction(ui->actionAccents_overleftrightarrow);
    toolAccents->addAction(ui->actionAccents_overrightarrow);
    toolAccents->addAction(ui->actionAccents_overline);
    toolAccents->addAction(ui->actionAccents_tildewide);
    toolAccents->addAction(ui->actionAccents_underbrace);
    toolAccents->addAction(ui->actionAccents_underleftarrow);
    toolAccents->addAction(ui->actionAccents_underrightarrow);
    toolAccents->addAction(ui->actionAccents_vector);

    toolOps = new WidgetToolMenu( 3, 1, this);
    toolOps->addAction(ui->actionOps_pm);
    toolOps->addAction(ui->actionOps_times);
    toolOps->addAction(ui->actionOps_div);
    toolOps->addAction(ui->actionOps_mp);
    toolOps->addAction(ui->actionOps_cap);
    toolOps->addAction(ui->actionOps_cup);
    toolOps->addAction(ui->actionOps_sqcap);
    toolOps->addAction(ui->actionOps_sqcup);
    toolOps->addAction(ui->actionOps_uplus);
    toolOps->addAction(ui->actionOps_amalg);
    toolOps->addAction(ui->actionOps_ominus);
    toolOps->addAction(ui->actionOps_oplus);
    toolOps->addAction(ui->actionOps_otimes);
    toolOps->addAction(ui->actionOps_oslash);
    toolOps->addAction(ui->actionOps_odot);
    toolOps->addAction(ui->actionOps_cdot);
    toolOps->addAction(ui->actionOps_circ);
    toolOps->addAction(ui->actionOps_bullet);
    toolOps->addAction(ui->actionOps_bigcirc);
    toolOps->addAction(ui->actionOps_star);
    toolOps->addAction(ui->actionOps_diamond);
    toolOps->addAction(ui->actionOps_vee);
    toolOps->addAction(ui->actionOps_wedge);
    toolOps->addAction(ui->actionOps_setminus);
    toolOps->addAction(ui->actionOps_wr);
    toolOps->addAction(ui->actionOps_dagger);
    toolOps->addAction(ui->actionOps_daggerd);
    toolOps->addAction(ui->actionOps_triangleleft);
    toolOps->addAction(ui->actionOps_triangleright);
    toolOps->addAction(ui->actionOps_bigtriangleup);
    toolOps->addAction(ui->actionOps_bigtriangledown);

    toolRelations = new WidgetToolMenu( 3, 1, this);
    toolRelations->addAction(ui->actionRel_leq);
    toolRelations->addAction(ui->actionRel_neq);
    toolRelations->addAction(ui->actionRel_simeq);
    toolRelations->addAction(ui->actionRel_geq);
    toolRelations->addAction(ui->actionRel_equiv);
    toolRelations->addAction(ui->actionRel_models);
    toolRelations->addAction(ui->actionRel_prec);
    toolRelations->addAction(ui->actionRel_succ);
    toolRelations->addAction(ui->actionRel_sim);
    toolRelations->addAction(ui->actionRel_perp);
    toolRelations->addAction(ui->actionRel_preceq);
    toolRelations->addAction(ui->actionRel_succeq);
    toolRelations->addAction(ui->actionRel_mid);
    toolRelations->addAction(ui->actionRel_lless);
    toolRelations->addAction(ui->actionRel_ggreater);
    toolRelations->addAction(ui->actionRel_asymp);
    toolRelations->addAction(ui->actionRel_parallel);
    toolRelations->addAction(ui->actionRel_subset);
    toolRelations->addAction(ui->actionRel_supset);
    toolRelations->addAction(ui->actionRel_approx);
    toolRelations->addAction(ui->actionRel_smile);
    toolRelations->addAction(ui->actionRel_subseteq);
    toolRelations->addAction(ui->actionRel_supseteq);
    toolRelations->addAction(ui->actionRel_cong);
    toolRelations->addAction(ui->actionRel_frown);
    toolRelations->addAction(ui->actionRel_sqsubseteq);
    toolRelations->addAction(ui->actionRel_sqsupseteq);
    toolRelations->addAction(ui->actionRel_dotequal);
    toolRelations->addAction(ui->actionRel_in);
    toolRelations->addAction(ui->actionRel_ni);
    toolRelations->addAction(ui->actionRel_propto);
    toolRelations->addAction(ui->actionRel_notin);
    toolRelations->addAction(ui->actionRel_vdash);
    toolRelations->addAction(ui->actionRel_dashv);
    toolRelations->addAction(ui->actionRel_bowtie);

    toolMisc = new WidgetToolMenu(3, 1, this);
    toolMisc->addAction(ui->actionMisc_infty);
    toolMisc->addAction(ui->actionMisc_forall);
    toolMisc->addAction(ui->actionMisc_exists);
    toolMisc->addAction(ui->actionMisc_nabla);
    toolMisc->addAction(ui->actionMisc_partial);
    toolMisc->addAction(ui->actionMisc_prime);
    toolMisc->addAction(ui->actionMisc_ell);
    toolMisc->addAction(ui->actionMisc_emptyset);
    toolMisc->addAction(ui->actionMisc_imath);
    toolMisc->addAction(ui->actionMisc_jmath);
    toolMisc->addAction(ui->actionMisc_wp);
    toolMisc->addAction(ui->actionMisc_textrmO);
    toolMisc->addAction(ui->actionMisc_hbar);

    toolSets = new WidgetToolMenu(3,1, this);
    toolSets->addAction(ui->actionSets_re);
    toolSets->addAction(ui->actionSets_mathbbZ);
    toolSets->addAction(ui->actionSets_mathcalH);
    toolSets->addAction(ui->actionSets_im);
    toolSets->addAction(ui->actionSets_mathbbC);
    toolSets->addAction(ui->actionSets_mathcalF);
    toolSets->addAction(ui->actionSets_aleph);
    toolSets->addAction(ui->actionSets_mathbbH);
    toolSets->addAction(ui->actionSets_mathcalL);
    toolSets->addAction(ui->actionSets_mathbbN);
    toolSets->addAction(ui->actionSets_mathbbQ);
    toolSets->addAction(ui->actionSets_mathcalO);
    toolSets->addAction(ui->actionSets_mathbbR);

    toolGreek = new WidgetToolMenu(3,1 , this);
    toolGreek->addAction(ui->actionGreek_alpha);
    toolGreek->addAction(ui->actionGreek_beta);
    toolGreek->addAction(ui->actionGreek_chi);
    toolGreek->addAction(ui->actionGreek_delta);
    toolGreek->addAction(ui->actionGreek_epsilon);
    toolGreek->addAction(ui->actionGreek_epsilonVar);
    toolGreek->addAction(ui->actionGreek_eta);
    toolGreek->addAction(ui->actionGreek_gamma);
    toolGreek->addAction(ui->actionGreek_iota);
    toolGreek->addAction(ui->actionGreek_kappa);
    toolGreek->addAction(ui->actionGreek_lambda);
    toolGreek->addAction(ui->actionGreek_mu);
    toolGreek->addAction(ui->actionGreek_nu);
    toolGreek->addAction(ui->actionGreek_omega);
    toolGreek->addAction(ui->actionGreek_phi);
    toolGreek->addAction(ui->actionGreek_phiVar);
    toolGreek->addAction(ui->actionGreek_pi);
    toolGreek->addAction(ui->actionGreek_piVar);
    toolGreek->addAction(ui->actionGreek_psi);
    toolGreek->addAction(ui->actionGreek_rho);
    toolGreek->addAction(ui->actionGreek_rhoVar);
    toolGreek->addAction(ui->actionGreek_sigma);
    toolGreek->addAction(ui->actionGreek_sigmaVar);
    toolGreek->addAction(ui->actionGreek_tau);
    toolGreek->addAction(ui->actionGreek_theta);
    toolGreek->addAction(ui->actionGreek_thetaVar);
    toolGreek->addAction(ui->actionGreek_upsilon);
    toolGreek->addAction(ui->actionGreek_xi);
    toolGreek->addAction(ui->actionGreek_zeta);

    toolGreekBig = new WidgetToolMenu(3, 1, this);
    toolGreekBig->addAction(ui->actionGreekBig_delta);
    toolGreekBig->addAction(ui->actionGreekBig_gamma);
    toolGreekBig->addAction(ui->actionGreekBig_lambda);
    toolGreekBig->addAction(ui->actionGreekBig_phi);
    toolGreekBig->addAction(ui->actionGreekBig_pi);
    toolGreekBig->addAction(ui->actionGreekBig_psi);
    toolGreekBig->addAction(ui->actionGreekBig_sigma);
    toolGreekBig->addAction(ui->actionGreekBig_theta);
    toolGreekBig->addAction(ui->actionGreekBig_upsilon);
    toolGreekBig->addAction(ui->actionGreekBig_xi);

    toolSums = new WidgetToolMenu( 3, 1, this);
    toolSums->addAction(ui->actionSum_sum);
    toolSums->addAction(ui->actionSum_sumArgs);
    toolSums->addAction(ui->actionSum_sumArgs2);
    
    toolIntegrals = new WidgetToolMenu(2,1, this);
    toolIntegrals->addAction(ui->actionInt_simple);
    toolIntegrals->addAction(ui->actionInt_oint);
    toolIntegrals->addAction(ui->actionInt_intDouble);
    toolIntegrals->addAction(ui->actionInt_oint2);
    toolIntegrals->addAction(ui->actionInt_int2);
    toolIntegrals->addAction(ui->actionInt_intTriple);
    toolIntegrals->addAction(ui->actionInt_intQuadruple);
    toolIntegrals->addAction(ui->actionInt_ointop);
    toolIntegrals->addAction(ui->actionInt_dots);

    toolProd = new WidgetToolMenu(2, 1, this);
    toolProd->addAction(ui->actionProd_prod);
    toolProd->addAction(ui->actionProd_coproduct);
    toolProd->addAction(ui->actionProd_prodArgs);
    toolProd->addAction(ui->actionProd_coproductArgs);
    toolProd->addAction(ui->actionProd_prodArgs2);
    toolProd->addAction(ui->actionProd_coproductArgs2);

    toolArrows = new WidgetToolMenu(3, 1, this);
    toolArrows->addAction(ui->actionArrow_rightarrow);
    toolArrows->addAction(ui->actionArrow_leftarrow);
    toolArrows->addAction(ui->actionArrow_downarrow);
    toolArrows->addAction(ui->actionArrow_uparrow);
    toolArrows->addAction(ui->actionArrow_updownarrow);
    toolArrows->addAction(ui->actionArrow_leftrightarrow);
    toolArrows->addAction(ui->actionArrow_leftrightarrow2);
    toolArrows->addAction(ui->actionArrow_leftarrow2);
    toolArrows->addAction(ui->actionArrow_rightarrow2);
    toolArrows->addAction(ui->actionArrow_downarrow2);
    toolArrows->addAction(ui->actionArrow_uparrow2);
    toolArrows->addAction(ui->actionArrow_updownarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow2);
    toolArrows->addAction(ui->actionArrow_longleftarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow);
    toolArrows->addAction(ui->actionArrow_longleftarrow);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow);
    toolArrows->addAction(ui->actionArrow_leftharpoondown);
    toolArrows->addAction(ui->actionArrow_rightharpoondown);
    toolArrows->addAction(ui->actionArrow_mapsto);
    toolArrows->addAction(ui->actionArrow_mapsto2);
    toolArrows->addAction(ui->actionArrow_arrowNW);
    toolArrows->addAction(ui->actionArrow_arrowNE);
    toolArrows->addAction(ui->actionArrow_leftharpoonup);
    toolArrows->addAction(ui->actionArrow_rightharpoonup);
    toolArrows->addAction(ui->actionArrow_hookleftarrow);
    toolArrows->addAction(ui->actionArrow_hookrightarrow);
    toolArrows->addAction(ui->actionArrow_arrowSW);
    toolArrows->addAction(ui->actionArrow_arrowSE);
    toolArrows->addAction(ui->actionArrow_rightleftharpoons);

    for (int i = 0; i < MAX_RECENT_FILES; ++i) {
        recentFileActions[i] = new QAction(this);
        recentFileActions[i]->setVisible(false);
        connect(recentFileActions[i], SIGNAL(triggered()),
                this, SLOT(openRecentFile()));
        ui->menu_File->insertAction(ui->actionRecents ,recentFileActions[i]);
    }

    // Set actions on Equation
    QAction* copyEquationToClipoard = new QAction(QIcon::fromTheme("edit-copy"), tr("Copy"), this);
    copyEquationToClipoard->setShortcut(QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_C)); // copy equation using CTRL+ALT+C
    connect(copyEquationToClipoard, SIGNAL(triggered()), this, SLOT(copyEquationToClipboard()) );

    QAction* pasteEquationFromClipoard = new QAction(QIcon::fromTheme("edit-paste"), tr("Paste"), this);
    pasteEquationFromClipoard->setShortcut(QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_V)); // copy equation using CTRL+ALT+V
    connect(pasteEquationFromClipoard, SIGNAL(triggered()), this, SLOT(pasteEquationFromClipboard()) );

    QAction* actSep1 = new QAction(this);
    actSep1->setSeparator(true);
    QAction* actSep2 = new QAction(this);
    actSep2->setSeparator(true);

    ui->equationArea->addAction(ui->actionView_Refresh);
    ui->equationArea->addAction(actSep1);
    ui->equationArea->addAction(copyEquationToClipoard);
    ui->equationArea->addAction(pasteEquationFromClipoard);
    ui->equationArea->addAction(actSep2);
    ui->equationArea->addAction(ui->actionView_StyleBar);


}

void MainWindow::setSymbolsPanel()
{

    QFrame *mSymGrid = new QFrame;
    QGridLayout *mSymGridLayout = new QGridLayout;
    mSymGridLayout->setSpacing(0);
    mSymGridLayout->setContentsMargins(0,0,0,0);

    mSymGrid->setLayout(mSymGridLayout);

    // first row
    mSymGridLayout->addWidget(toolDeco, 0, 0);
    mSymGridLayout->addWidget(toolFunc, 0, 1);
    mSymGridLayout->addWidget(toolDelim, 0, 2);
    mSymGridLayout->addWidget(toolAccents, 0, 3);
    mSymGridLayout->addWidget(toolOps, 0, 4);
    mSymGridLayout->addWidget(toolRelations, 0, 5);
    mSymGridLayout->addWidget(toolMisc, 0, 6);

    // second row
    mSymGridLayout->addWidget(toolGreek, 1, 0);
    mSymGridLayout->addWidget(toolGreekBig, 1, 1);
    mSymGridLayout->addWidget(toolSums, 1, 2);
    mSymGridLayout->addWidget(toolIntegrals, 1, 3);
    mSymGridLayout->addWidget(toolProd, 1, 4);
    mSymGridLayout->addWidget(toolSets, 1, 5);
    mSymGridLayout->addWidget(toolArrows, 1, 6);

    // add to Symbols ToolBar
    ui->symbolsBar->addWidget(mSymGrid);

}



void MainWindow::addButtonToMathTabs(const QStringList &list, QWidget *tabWidget)
{
    QHBoxLayout * tabLayout = (QHBoxLayout *)tabWidget->layout();

    if(!tabLayout){
        tabLayout = new QHBoxLayout;
    }

    EquationTemplateWidget *eqTemplate = new EquationTemplateWidget(list, tabWidget);

    connect(eqTemplate, SIGNAL(clicked(QString)), editor, SLOT(replace(QString)) );

    tabLayout->addWidget(eqTemplate);
    tabLayout->addStretch(0);

    tabLayout->setSpacing(0);
    tabLayout->setContentsMargins(0,0,0,0);

    tabWidget->setLayout(tabLayout);

}

void MainWindow::setMathPanelTabsActions()
{
    signalMapper = new QSignalMapper(this);

    // Algebra Tab
    QStringList listAlgebra;
    listAlgebra << ":/resources/icons/tabBar/algebra/eq01.png"
                << ":/resources/icons/tabBar/algebra/eq02.png"
                << ":/resources/icons/tabBar/algebra/eq03.png"
                << ":/resources/icons/tabBar/algebra/eq04.png"
                << ":/resources/icons/tabBar/algebra/eq05.png"
                << ":/resources/icons/tabBar/algebra/eq06.png" ;

    addButtonToMathTabs(listAlgebra,templateTab->getPage(0) );


    // Derivs Tab
    QStringList listDerivs;
    listDerivs << ":/resources/icons/tabBar/derivs/eq01.png"
               << ":/resources/icons/tabBar/derivs/eq02.png"
               << ":/resources/icons/tabBar/derivs/eq03.png"
               << ":/resources/icons/tabBar/derivs/eq04.png"
               << ":/resources/icons/tabBar/derivs/eq05.png"
               << ":/resources/icons/tabBar/derivs/eq06.png" ;

    addButtonToMathTabs(listDerivs, templateTab->getPage(1));

    // Stats Tab
    QStringList listStats;
    listStats << ":/resources/icons/tabBar/stats/eq01.png"
              << ":/resources/icons/tabBar/stats/eq02.png"
              << ":/resources/icons/tabBar/stats/eq03.png"
              << ":/resources/icons/tabBar/stats/eq04.png"
              << ":/resources/icons/tabBar/stats/eq05.png"
              << ":/resources/icons/tabBar/stats/eq06.png"
              << ":/resources/icons/tabBar/stats/eq07.png" ;

    addButtonToMathTabs(listStats, templateTab->getPage(2));

    // Matrix Tab
    QStringList listMatrix;
    listMatrix << ":/resources/icons/tabBar/matrix/eq01.png"
               << ":/resources/icons/tabBar/matrix/eq02.png"
               << ":/resources/icons/tabBar/matrix/eq03.png"
               << ":/resources/icons/tabBar/matrix/eq04.png";

    addButtonToMathTabs(listMatrix, templateTab->getPage(3));

    // Sets Tab
    QStringList listSets;
    listSets << ":/resources/icons/tabBar/sets/eq01.png"
             << ":/resources/icons/tabBar/sets/eq02.png";


    addButtonToMathTabs(listSets, templateTab->getPage(4));

    // Trig Tab
    QStringList listTrig;
    listTrig << ":/resources/icons/tabBar/trig/eq01.png"
             << ":/resources/icons/tabBar/trig/eq02.png"
             << ":/resources/icons/tabBar/trig/eq03.png"
             << ":/resources/icons/tabBar/trig/eq04.png";

    addButtonToMathTabs(listTrig, templateTab->getPage(5));

    // Geometry Tab
    QStringList listGeometry;
    listGeometry << ":/resources/icons/tabBar/geometry/eq01.png"
                 << ":/resources/icons/tabBar/geometry/eq02.png"
                 << ":/resources/icons/tabBar/geometry/eq03.png"
                 << ":/resources/icons/tabBar/geometry/eq04.png";

    addButtonToMathTabs(listGeometry, templateTab->getPage(6));

    // Chemistry Tab
    QStringList listChemistry;
    listChemistry << ":/resources/icons/tabBar/chemistry/eq01.png"
                  << ":/resources/icons/tabBar/chemistry/eq02.png"
                  << ":/resources/icons/tabBar/chemistry/eq03.png"
                  << ":/resources/icons/tabBar/chemistry/eq04.png"
                  << ":/resources/icons/tabBar/chemistry/eq05.png";

    addButtonToMathTabs(listChemistry, templateTab->getPage(7));

    // Physics Tab
    QStringList listPhysics;
    listPhysics << ":/resources/icons/tabBar/physics/eq01.png"
                << ":/resources/icons/tabBar/physics/eq02.png"
                << ":/resources/icons/tabBar/physics/eq03.png"
                << ":/resources/icons/tabBar/physics/eq04.png";

    addButtonToMathTabs(listPhysics, templateTab->getPage(8));


    ui->templatesBar->addWidget(templateTab);

}

void MainWindow::setConverters()
{
    EqualX::Converter c;
    c.path = preferencesDialog->pdfCairoEdit->text();
    c.opts = "-png -transp -singlefile %1 %2";
    renderer->addConverter("png", c);

    EqualX::Converter c2;
    c2.path = preferencesDialog->pdfCairoEdit->text();
    c2.opts = "-svg %1 %2.svg";
    renderer->addConverter("svg", c2);

    EqualX::Converter c3;
    c3.path = preferencesDialog->pdfCairoEdit->text();
    c3.opts = "-jpeg -singlefile %1 %2";
    renderer->addConverter("jpg", c3);

    EqualX::Converter c4;
    c4.path = preferencesDialog->pdfCairoEdit->text();
    c4.opts = "-ps -level2 %1 %2.ps"; //! need level 2 to insert xmp metadata
    renderer->addConverter("ps", c4);

    EqualX::Converter c5;
    c5.path = preferencesDialog->pdfCairoEdit->text();
    c5.opts = "-eps %1 %2.eps";
    renderer->addConverter("eps", c5);

}

void MainWindow::setSignals()
{
    /* MainWindow */
    // Menu Bar
    // File Menu
    connect(ui->actionNew, SIGNAL(triggered()), this, SLOT(newEquation()) );
    connect(ui->actionOpen, SIGNAL(triggered()), this, SLOT(open()) );
    connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(save()) );
    connect(ui->actionSaveAs, SIGNAL(triggered()), this, SLOT(saveAs()) );
    connect(ui->actionQuit, SIGNAL(triggered()), this, SLOT(close()) );

    // Edit Menu
    connect(ui->actionEdit_Preferences, SIGNAL(triggered()), this, SLOT(onPreferences()) );
    connect(ui->actionEdit_Delete, SIGNAL(triggered()), this, SLOT(onDeleteSelection()) );

    // View Menu
    connect(ui->actionView_Refresh, SIGNAL(triggered()), this, SLOT(updateEquation()) );

    // Search Menu
    connect(ui->actionFind, SIGNAL(triggered()), this, SLOT(showFindDialog()));
    connect(ui->actionFind_Next, SIGNAL(triggered()), editor, SLOT(findNext()) );
    connect(ui->actionFind_Previous, SIGNAL(triggered()), editor, SLOT(findPrevious()) );
    connect(ui->actionReplace, SIGNAL(triggered()), this, SLOT(showReplaceDialog()) );
    connect(findWidget, SIGNAL(closing()), editor, SLOT(clearSelections()) );
    connect(replaceDialog, SIGNAL(closing()), editor, SLOT(clearSelections()) );

    // Help Menu
    connect(ui->actionHelp_About, SIGNAL(triggered()), this, SLOT(onAbout()) );
    connect(ui->actionAboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()) );

    // GUI
    connect(editor, SIGNAL(textChanged()), this, SLOT(equationChanged()) );
    connect(ui->editPreamble, SIGNAL(blockCountChanged(int)), editor, SLOT(setStartCountingBlocks(int)) );
    connect(ui->editPreamble, SIGNAL(textChanged()), this, SLOT(equationChanged()) );
    connect(ui->renderButton, SIGNAL(released()), this, SLOT(updateEquation()) );
    connect(renderModeButtonGroup, SIGNAL(buttonClicked(int)), this, SLOT(equationChanged()) );
    connect(ui->equationArea, SIGNAL(dropped(QString)), this, SLOT(loadEquation(QString)) );

    connect(ui->fgColorPicker, SIGNAL(colorSelectionChanged(QColor)), this, SLOT(equationChanged()) );
    connect(ui->bgColorPicker, SIGNAL(colorSelectionChanged(QColor)), this, SLOT(equationChanged()) );
    connect(ui->fontSizeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(equationChanged()) );

    // Find widget
    connect(findWidget, SIGNAL(changedFindText(QString)), this, SLOT(onFindTextChanged(QString))    );
    connect(findWidget, SIGNAL(find(QString,QTextDocument::FindFlags)), editor, SLOT(find(QString,QTextDocument::FindFlags)) );
    connect(findWidget, SIGNAL(findAll(QString,QTextDocument::FindFlags)), editor, SLOT(findAll(QString,QTextDocument::FindFlags)) );

    // Replace Dialog
    connect(replaceDialog, SIGNAL(find(QString,QTextDocument::FindFlags)), editor, SLOT(findAll(QString,QTextDocument::FindFlags)) );
    connect(replaceDialog, SIGNAL(replace(QString)), editor, SLOT(replace(QString)) );
    connect(replaceDialog, SIGNAL(replaceAll(QString)), editor, SLOT(replaceAll(QString)) );

    // Symbols Toolbar
    connect(toolDeco, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolFunc, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolDelim, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolAccents, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolOps, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolRelations, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolMisc, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolSets, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolGreek, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolGreekBig, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolSums, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolIntegrals, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolProd, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolArrows, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );

    // Other
    connect(preferencesDialog, SIGNAL(accepted()), this, SLOT(preferencesAccepted()) );
    connect(preferencesDialog->renderManualUpdate, SIGNAL(toggled(bool)), ui->renderButton, SLOT(setVisible(bool)) );

    connect(renderer, SIGNAL(started()), this, SLOT(renderStarted()) );
    connect(renderer, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(renderFinished(int,QProcess::ExitStatus)) );
    connect(ui->fontSizeComboBox, SIGNAL(currentIndexChanged(int)) , equation, SLOT(setFontSize(int)) );
    connect(ui->fgColorPicker, SIGNAL(colorSelectionChanged(QColor)), equation, SLOT(setForegroundColor(QColor)) );
    connect(ui->bgColorPicker, SIGNAL(colorSelectionChanged(QColor)), equation, SLOT(setBackgroundColor(QColor)) );
    connect(renderModeButtonGroup, SIGNAL(buttonClicked(int)), equation, SLOT(setEnvironment(int)) );

    connect(&mAutoUpdateTimer, SIGNAL(timeout()), this, SLOT(updateEquation()) );
}


void MainWindow::closeEvent(QCloseEvent *event)
{
    writeSettings();

    if ( wishToSave()) save();

    QMainWindow::closeEvent(event);
}

void MainWindow::showEquation()
{
    const QMap<int,QString> *errors = renderer->latexErrors();

    bool latexHasErrors = !errors->isEmpty();
    if(!editor->hasFocus()) editor->setFocus();

    if(!latexHasErrors){  // do not try to parse the string for errors when there are none
        editor->selectLines(*errors); // clears any highlightings
        ui->editPreamble->selectLines(*errors); // clears any highlightings
        ui->stackedWidget->setCurrentIndex(0);
        equationView->setPixmap( renderer->file("png") );
        equationView->setExportSource(renderer->file(preferencesDialog->exportCBox->currentText()));
    }
    else{// show the Errors page
        ui->stackedWidget->setCurrentIndex(1);

        QMapIterator<int, QString> i(*errors);
        QList<QTreeWidgetItem *> items;
        QStringList row;
        while (i.hasNext()) {
            i.next();
            row.clear();
            row << QString("%1").arg(i.key())<< i.value();
            items.append(new QTreeWidgetItem((QTreeWidget*)0, row));
        }

        ui->treeWidgetErrors->clear();
        ui->treeWidgetErrors->insertTopLevelItems(0, items);
        editor->selectLines(*errors);
        ui->editPreamble->selectLines(*errors);
    }

    setWindowModified(!latexHasErrors);

}

void MainWindow::newEquation()
{
    if (wishToSave()) save();

    QFile f(":/resources/templates/template-preamble.tex");
    f.open(QIODevice::ReadOnly);
    QString preambleContent = f.readAll();
    f.close();

    ui->editPreamble->setPlainText(preambleContent);
    editor->clear();
    equationView->clear();

    updateUI();
    setCurrentFile("");


}

void MainWindow::open()
{
    QString curDirToSave;

    if (wishToSave()) save();

    if( curFile.isEmpty() )
        curDirToSave = QDir::homePath();
    else
        curDirToSave = QFileInfo(curFile).dir().path();

    QString filter = renderer->fileTypes().join("|");
    filter.replace("|", ");;(*.");
    filter = "(*."+filter+")";

    QString allTypesFilter = renderer->fileTypes().join(" *.");
    filter = tr("All Supported files (*.")+allTypesFilter+");;"+filter;

    QString fileName = QFileDialog::getOpenFileName(this, tr("Open Equation"),
                                                    curDirToSave,
                                                    filter);
    if (!fileName.isEmpty())
        loadEquation(fileName);

}

void MainWindow::openRecentFile()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action){
        loadEquation(action->data().toString());
    }
}

bool MainWindow::save()
{
    if (curFile.isEmpty()) {
        return saveAs();
    } else {
        return saveEquation(curFile);
    }
}

bool MainWindow::saveAs()
{
    QString curDirToSave;

    if( curFile.isEmpty() )
        curDirToSave = QDir::homePath();
    else
        curDirToSave = QFileInfo(curFile).dir().path();

    QString filter = renderer->fileTypes().join(";;");
    filter.replace(";;", ");;(*.");
    filter = "(*."+filter+")";

    QString selectedFilter;
    selectedFilter = "(*."+preferencesDialog->exportCBox->currentText()+")";

    QString fileName = QFileDialog::getSaveFileName(0, tr("Save Equation as..."), curDirToSave, filter, &selectedFilter );

    if (fileName.isEmpty())
        return false;


    int s = selectedFilter.length()-3;
    QString fileExt = selectedFilter.mid(2,s);
    fileName+=fileExt;

    return saveEquation(fileName);
}

void MainWindow::preferencesAccepted()
{
    writeSettings();

    updateUI();
    updateRecentFileActions();
}

bool MainWindow::saveEquation(const QString &fileName)
{
    qDebug() << "[MainWindow::saveEquation] Saving equation to: " << fileName;

    QString ext = fileName.section(".", -1);

    renderer->run(ext);

    QFile::remove(fileName);
    QFile::copy(renderer->file(ext), fileName);

    setCurrentFile(fileName);

    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);
    QStringList files = settings.value("RecentFileList").toStringList();
    files.removeAll(fileName);
    files.prepend(fileName);
    while (files.size() > MAX_RECENT_FILES)
        files.removeLast();

    settings.setValue("RecentFileList", files);

    updateRecentFileActions();

    return true;
}

void MainWindow::onDeleteSelection()
{
    editor->textCursor().removeSelectedText();
}

bool MainWindow::onPreferences()
{
    preferencesDialog->show();

    return true;
}

void MainWindow::onAbout()
{

    QFile file(":/LICENSE");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&file);
    QString licenseText = in.readAll();
    in.flush();


    aboutDialog->setLicenseText(licenseText);
    aboutDialog->show();

}

// read all settings from the Settings File and update preferencesDialog
void MainWindow::readSettings()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);

    settings.beginGroup("MAIN");
    preferencesDialog->recentEqNumber->setValue(settings.value("max_recent", DEFAULT_MAX_RECENT_FILES ).toInt() );
    preferencesDialog->checkRememberLayout->setChecked( settings.value("remember_layout", DEFAULT_REMEMBER_LAYOUT).toBool() );

    if( preferencesDialog->checkRememberLayout->isChecked() ){
        ui->actionView_MenuBar->setChecked( settings.value("mainbar", DEFAULT_SHOW_MAIN_TOOLBAR).toBool() );
        ui->actionView_SymbolsBar->setChecked( settings.value("panel_symbols", DEFAULT_SHOW_PANEL_SYMBOLS).toBool() );
        ui->actionView_TemplatesBar->setChecked( settings.value("panel_templates", DEFAULT_SHOW_PANEL_TEMPLATES).toBool() );
        ui->actionView_StyleBar->setChecked(settings.value("panel_properties", DEFAULT_SHOW_PANEL_PROPERTIES).toBool() );

        QPoint pos = settings.value("position", QPoint(DEFAULT_WINDOW_X, DEFAULT_WINDOW_Y)).toPoint();
        QSize size = settings.value("size", QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT)).toSize();
        resize(size);
        move(pos);
    }
    preferencesDialog->exportCBox->setCurrentIndex(settings.value("export", DEFAULT_EXPORT_TYPE).toInt() );
    settings.endGroup();

    settings.beginGroup("EDITOR");
    preferencesDialog->comboFontFamily->setCurrentIndex(settings.value("font", DEFAULT_FONT).toInt() );
    preferencesDialog->comboFontSize->setCurrentIndex( settings.value("font_size", DEFAULT_FONT_SIZE).toInt() );
    preferencesDialog->checkSyntaxHighligthing->setChecked(settings.value("highlighting", DEFAULT_HIGHLIGHTING ).toBool());
    preferencesDialog->checkTextWrapping->setChecked( settings.value("textwrap", DEFAULT_WRAPPING).toBool() );
    preferencesDialog->groupBoxCompletion->setChecked( settings.value("completion", DEFAULT_COMPLETION).toBool() );
    preferencesDialog->checkSensitiveCompletion->setChecked( settings.value("case_sensitive_completion", DEFAULT_COMPLETION).toBool() );
    settings.endGroup();

    settings.beginGroup("PREVIEW");
    preferencesDialog->renderAutoUpdate->setChecked( settings.value("updateAuto", DEFAULT_UPDATE_AUTO ).toBool() );
    preferencesDialog->renderManualUpdate->setChecked(! settings.value("updateAuto", DEFAULT_UPDATE_AUTO ).toBool() );
    preferencesDialog->spinUpdateTime->setValue(settings.value("updateTime", DEFAULT_UPDATE_TIME ).toInt());
    preferencesDialog->bgPreviewColor->setColorName(settings.value("bgcolor", DEFAULT_PREVIEW_BG).toString() );
    preferencesDialog->fgColorPicker->setColorName( settings.value("render_fgcolor", DEFAULT_RENDER_FG).toString() );
    preferencesDialog->bgColorPicker->setColorName( settings.value("render_bgcolor", DEFAULT_RENDER_BG).toString() );
    preferencesDialog->renderFontSize->setCurrentIndex( settings.value("render_fontsize", DEFAULT_RENDER_FONT_SIZE).toInt() );
    switch(settings.value("render_mode", DEFAULT_RENDER_MODE).toInt()){
    case LATEX_ENV_MODE_INLINE:
        preferencesDialog->renderModeInline->setChecked(true);
        break;
    case LATEX_ENV_MODE_ALIGN:
        preferencesDialog->renderModeAlign->setChecked(true);
        break;
    case LATEX_ENV_MODE_TEXT:
        preferencesDialog->renderModeText->setChecked(true);
        break;
    default:
    case LATEX_ENV_MODE_DISPLAY:
        preferencesDialog->renderModeDisplay->setChecked(true);
        break;
    }
    settings.endGroup();

    settings.beginGroup("CONVERTERS");
    preferencesDialog->pdfLatexEdit->setText(settings.value("pdflatex", DEFAULT_PDFLATEX ).toString());
    preferencesDialog->pdfCairoEdit->setText(settings.value("pdftocairo", DEFAULT_PDFCAIRO ).toString());
    preferencesDialog->gsEdit->setText(settings.value("gs", DEFAULT_GS ).toString());
    settings.endGroup();
}

void MainWindow::renderStarted()
{
    ui->stackedWidget->setCurrentIndex(0);

    equationView->setMovie(movieWorkInProgress);
    movieWorkInProgress->start();

    QApplication::setOverrideCursor(Qt::BusyCursor);
}

void MainWindow::renderFinished( int exitCode, QProcess::ExitStatus exitStatus)
{
    qDebug() << "[Render Finished] exitCode:"<<exitCode<<" exitStatus:"<<exitStatus;
    movieWorkInProgress->stop();
    setEnabled(true);
    QApplication::restoreOverrideCursor();

    if(exitCode!=0 || exitStatus != QProcess::NormalExit){
        QMessageBox::critical(this, "Error - EqualX", QString("Command failed to run. %1").arg("err") );

        //return;
    }

    showEquation();
}

void MainWindow::updateEquation()
{

    qDebug() << "========== Update Equation =============";

    equationView->clear();
    if(editor->toPlainText().isEmpty()){ // clear the view and do not launch renderer
        return;
    }

    setEnabled(false);

    equation->setSelections(editor->getSelections());
    equation->setPreamble(ui->editPreamble->toPlainText());
    equation->setEquation(editor->toPlainText());

    QStringList exportTypes;
    exportTypes << "png" << preferencesDialog->exportCBox->currentText();

    renderer->setFileInfo(*equation);
    renderer->run(exportTypes);

}

void MainWindow::updateUI()
{
    editor->selectAll();
    editor->setHighLighting( preferencesDialog->checkSyntaxHighligthing->isChecked() );
    editor->setCompletion( preferencesDialog->groupBoxCompletion->isChecked() );
    editor->setCompletionSensitive( preferencesDialog->checkSensitiveCompletion->isChecked() );
    editor->setFont(preferencesDialog->getFont());
    ui->editPreamble->setFont(preferencesDialog->getFont());

    if( preferencesDialog->checkTextWrapping->isChecked() )
        editor->setLineWrapMode(QPlainTextEdit::WidgetWidth);
    else
        editor->setLineWrapMode(QPlainTextEdit::NoWrap);


    ui->renderButton->setVisible( preferencesDialog->renderManualUpdate->isChecked() );

    QColor previewCol = preferencesDialog->bgPreviewColor->getColor();
    QString previewStyleSheet;
    if(previewCol.alpha()!=0)  previewStyleSheet = QString("QScrollArea {background-color: %1} QLabel {background-color: %1}").arg(previewCol.name());
    ui->equationArea->setStyleSheet( previewStyleSheet );

    ui->fontSizeComboBox->setCurrentIndex( preferencesDialog->renderFontSize->currentIndex() );
    ui->fgColorPicker->setColorName( preferencesDialog->fgColorPicker->getColorName() );
    ui->bgColorPicker->setColorName( preferencesDialog->bgColorPicker->getColorName() );

    switch(preferencesDialog->renderMode()){
    case LATEX_ENV_MODE_INLINE:
        ui->renderModeInline->setChecked(true);
        break;
    case LATEX_ENV_MODE_ALIGN:
        ui->renderModeAlign->setChecked(true);
        break;
    case LATEX_ENV_MODE_TEXT:
        ui->renderModeText->setChecked(true);
        break;
    default:
    case LATEX_ENV_MODE_DISPLAY:
        ui->renderModeDisplay->setChecked(true);
        break;
    }

    mAutoUpdateTimer.setInterval(preferencesDialog->spinUpdateTime->value());
    renderer->setPDFLatex(preferencesDialog->pdfLatexEdit->text());
    renderer->setGS(preferencesDialog->gsEdit->text());
}

void MainWindow::updateRecentFileActions()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);
    QStringList files = settings.value("RecentFileList").toStringList();

    if(files.isEmpty() || files.size() == 0 )
        return ;

    int numRecentFiles = qMin( preferencesDialog->recentEqNumber->value(), files.size() );
    for (int i = 0; i < numRecentFiles; ++i) {
        QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
        recentFileActions[i]->setText(text);
        recentFileActions[i]->setData(files[i]);
        recentFileActions[i]->setVisible(true);
    }

    for (int j = numRecentFiles; j < MAX_RECENT_FILES; ++j)
        recentFileActions[j]->setVisible(false);

}

// read all settings from preferencesDialog and write them to the Settings File
void MainWindow::writeSettings()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);

    settings.beginGroup("MAIN");
    settings.setValue("max_recent", preferencesDialog->recentEqNumber->value() );
    settings.setValue("remember_layout", preferencesDialog->checkRememberLayout->isChecked() );
    settings.setValue("position", pos() );
    settings.setValue("size", size());
    settings.setValue("mainbar", ui->mainBar->isVisible() );
    settings.setValue("panel_properties", ui->renderBar->isVisible() );
    settings.setValue("panel_symbols", ui->symbolsBar->isVisible() );
    settings.setValue("panel_templates", ui->templatesBar->isVisible() );
    settings.setValue("export", preferencesDialog->exportCBox->currentIndex() );
    settings.endGroup();

    settings.beginGroup("EDITOR");
    settings.setValue("font", preferencesDialog->comboFontFamily->currentIndex() );
    settings.setValue("font_size", preferencesDialog->comboFontSize->currentIndex() );
    settings.setValue("highlighting", preferencesDialog->checkSyntaxHighligthing->isChecked() );
    settings.setValue("textwrap", preferencesDialog->checkTextWrapping->isChecked() );
    settings.setValue("completion", preferencesDialog->groupBoxCompletion->isChecked() );
    settings.setValue("case_sensitive_completion", preferencesDialog->checkSensitiveCompletion->isChecked() );
    settings.endGroup();

    settings.beginGroup("PREVIEW");
    settings.setValue("updateAuto", preferencesDialog->renderAutoUpdate->isChecked() );
    settings.setValue("updateTime", preferencesDialog->spinUpdateTime->value() );
    settings.setValue("bgcolor", preferencesDialog->bgPreviewColor->getColorName() );
    settings.setValue("render_fgcolor", preferencesDialog->fgColorPicker->getColorName() );
    settings.setValue("render_bgcolor", preferencesDialog->bgColorPicker->getColorName() );
    settings.setValue("render_fontsize", preferencesDialog->renderFontSize->currentIndex() );
    settings.setValue("LATEX_ENV_MODE", preferencesDialog->renderMode() );
    settings.endGroup();

    settings.beginGroup("CONVERTERS");
    settings.setValue("pdflatex", preferencesDialog->pdfLatexEdit->text() );
    settings.setValue("pdfcairo", preferencesDialog->pdfCairoEdit->text() );
    settings.setValue("gs", preferencesDialog->gsEdit->text() );
    settings.endGroup();

}

void MainWindow::equationChanged()
{
    if(preferencesDialog->renderAutoUpdate->isChecked())
        mAutoUpdateTimer.start();

}

void MainWindow::onFindTextChanged(const QString &exp)
{
    // if we have a expression to search
    bool findable = (! exp.isEmpty()) && !exp.isNull();

    // Enable/Disable widgets from Search Menu accordingly
    ui->actionFind_Next->setEnabled(findable);
    ui->actionFind_Previous->setEnabled(findable);

    editor->setTextCursor( QTextCursor(editor->document()) );

}

void MainWindow::showFindDialog()
{
    findWidget->setFindExpr( editor->selectedText() );
    findWidget->show();

    editor->setTextCursor( QTextCursor(editor->document()) );
}

void MainWindow::showReplaceDialog()
{
    replaceDialog->setFindExpr( editor->selectedText() );
    replaceDialog->show();

    editor->setTextCursor( QTextCursor(editor->document()) );

}

void MainWindow::insertSymbol(QAction *action)
{
    editor->insertPlainText(action->text() );
}

void MainWindow::copyEquationToClipboard()
{
    QClipboard *clipboard = QApplication::clipboard();

    QList<QUrl> urls;
    urls.append( QUrl::fromLocalFile(equationView->exportSource()) );

    QMimeData *mimeData = new QMimeData();
    mimeData->setUrls(urls);

    /* special handling case for nautilus */
    QString urlForNautilus("copy\nFile://");
    urlForNautilus.append(equationView->exportSource());
    mimeData->setData("x-special/gnome-copied-files", urlForNautilus.toLatin1());

    clipboard->setMimeData(mimeData);
}

void MainWindow::pasteEquationFromClipboard()
{
    QClipboard *clipboard = QApplication::clipboard();
    const QMimeData *mimeData = clipboard->mimeData();

    if(!mimeData) return;

    if(mimeData->hasUrls()){
        QUrl url = mimeData->urls().at(0);
        if(!url.isLocalFile()) return;

        QString filename = url.toLocalFile();

        if(wishToSave()) save();
        loadEquation(filename);
    }
}

bool MainWindow::wishToSave()
{
    bool goAndSave = false;

    if ( isWindowModified () ) {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::question(this, APP_FULL_NAME,
                                    tr("The Equation has been modified.\n"
                                       "Do you want to save your changes?"),
                                    QMessageBox::Save | QMessageBox::Discard);
        if (ret == QMessageBox::Save)
            goAndSave = true;

    }

    return goAndSave;

}

void MainWindow::loadEquation(const QString &fileName)
{
    qDebug() << "[MainWindow::loadEquation] Loading file:"<< fileName;

    bool status = EqualX::File::fetchInfo(fileName, equation);

    if(!status){
        QMessageBox::warning(this, APP_FULL_NAME,
                             tr("Can not load file: %1.\n"
                                "Are you sure this file was created with EqualX?").arg(fileName),
                             QMessageBox::Ok);

        return;
    }

    editor->setPlainText(equation->equation());
    ui->editPreamble->setPlainText(equation->preamble());
    ui->fgColorPicker->setColor(equation->fgColor());
    ui->bgColorPicker->setColor(equation->bgColor());
    ui->fontSizeComboBox->setCurrentIndex(equation->fontSize());
    switch(equation->environment()){
    case LATEX_ENV_MODE_INLINE:
        ui->renderModeInline->setChecked(true);
        break;
    case LATEX_ENV_MODE_ALIGN:
        ui->renderModeAlign->setChecked(true);
        break;
    case LATEX_ENV_MODE_TEXT:
        ui->renderModeText->setChecked(true);
        break;
    default:
    case LATEX_ENV_MODE_DISPLAY:
        ui->renderModeDisplay->setChecked(true);
        break;
    }

    setCurrentFile(fileName);

    editor->textCursor().clearSelection();
    ui->editPreamble->textCursor().clearSelection();


    if(!mAutoUpdateTimer.isActive()) updateEquation();
}

void MainWindow::setCurrentFile(const QString &fileName)
{
    curFile = fileName;
    setWindowFilePath(curFile);

    editor->document()->setModified(false);
    setWindowModified(false);

    QString shownName;
    if (curFile.isEmpty())
        shownName = tr("Untitled Equation");
    else
        shownName = strippedName(curFile);

    setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(APP_FULL_NAME));
}

QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}

void MainWindow::checkRequirements()
{
    bool reqsAreMeet = true;
    bool checkPdfLatex, checkPdfCairo, checkGS;

    // highlight style sheet for paths with problem
    QString errorStyleSheet = "color: rgb(0, 0, 0);background-color: rgb(255, 0, 0);";
    QString reqMessage = tr("EqualX can not find the applications it needs. Please set in the Preferences the correct paths to let EqualX know where it can find the required applications.\n\n"
                            "Click Ignore to continue using EqualX without LaTeX support. You can set later these by going to Edit->Preferences->Commands.\n"
                            "Click OK to launch now the Preferences window."
                            );


    // check for pdflatex
    checkPdfLatex = QFile::exists(preferencesDialog->pdfLatexEdit->text());
    if(!checkPdfLatex) {preferencesDialog->pdfLatexEdit->setStyleSheet(errorStyleSheet);} else { preferencesDialog->pdfLatexEdit->setStyleSheet(""); }

    // check for pdftocairo
    checkPdfCairo = QFile::exists(preferencesDialog->pdfCairoEdit->text());
    if(!checkPdfCairo) {preferencesDialog->pdfCairoEdit->setStyleSheet(errorStyleSheet);} else { preferencesDialog->pdfCairoEdit->setStyleSheet(""); }

    // check for ghostscript
    checkGS = QFile::exists(preferencesDialog->gsEdit->text());
    if(!checkGS) {preferencesDialog->gsEdit->setStyleSheet(errorStyleSheet);} else { preferencesDialog->gsEdit->setStyleSheet(""); }


    // Requirements are meet if all checks are true
    reqsAreMeet = checkPdfLatex && checkPdfCairo && checkGS;

    if(!reqsAreMeet) {
        int response = QMessageBox::critical(this, tr("Error - EqualX"), reqMessage, QMessageBox::Ok, QMessageBox::Ignore);

        if(response == QMessageBox::Ignore) return;
        if(response == QMessageBox::Ok){
            preferencesDialog->show();
            preferencesDialog->tabWidget->setCurrentIndex(3);
        }

    }
}
